package cc.smtweb.framework.core.db.impl;

import cc.smtweb.framework.core.db.cache.ModelTableCache;
import cc.smtweb.framework.core.db.vo.ModelField;
import cc.smtweb.framework.core.db.vo.ModelTable;
import org.apache.commons.lang3.StringUtils;

import java.io.Serializable;

/**
 * Created by Akmm at 14-1-2 上午11:19
 * 实体bean抽象类
 */
public class DefaultEntity extends BaseBean implements Serializable, Cloneable {
    //表名
    private String _def_table_name;

    public DefaultEntity() {}

    public DefaultEntity(String _def_table_name) {
        this._def_table_name = _def_table_name;
    }

    public ModelTable getModelTable() {
        return ModelTableCache.getInstance().getByName(_def_table_name);
    }

    //根据实体定义，设默认值
    public void init() {
        ModelTable entity = getModelTable();
        if (entity == null) return;
        String pkField = getPkFieldName();
        for (ModelField field : entity.getFields()) {
            if (data.containsKey(field.getName())) continue;//有值了，不能动
            if (field.getName().equalsIgnoreCase(pkField)) continue;//是pk，不要初始化
            String s = field.getDefaultValue();
            if (StringUtils.isNotEmpty(s)) put(field.getName(), s);
        }
    }

    /** 主键字段 */
    public String getPkFieldName() {
        return getModelTable().getIdField();
    }

    public long getEntityId() {
        return getLong(getPkFieldName());
    }

    public void setEntityId(long id) {
        data.put(getPkFieldName(), id);
    }

    public boolean isNew() {
        return getEntityId() <= 0L;
    }

    @Override
    public DefaultEntity clone() throws CloneNotSupportedException {
        DefaultEntity bean = (DefaultEntity) super.clone();
        bean.getData().remove(getPkFieldName());
        return bean;
    }
}
