package cc.smtweb.framework.core.mvc.service;

import cc.smtweb.framework.core.SwException;
import cc.smtweb.framework.core.cache.AbstractCache;
import cc.smtweb.framework.core.cache.CacheManager;
import cc.smtweb.framework.core.common.SwConsts;
import cc.smtweb.framework.core.common.SwEnum;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.EntityDao;
import cc.smtweb.framework.core.db.cache.ModelTableCache;
import cc.smtweb.framework.core.db.impl.DefaultEntity;
import cc.smtweb.framework.core.db.vo.ModelField;
import cc.smtweb.framework.core.db.vo.ModelTable;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Comparator;
import java.util.List;

/**
 * Created by Akmm at 2022/3/17 19:56
 */
public class TreeHelper<T extends DefaultEntity> {
    private String tableName;

    private ModelTable table;
    private AbstractCache<T> cache;
    private EntityDao<T> dao;

    private String fieldParent;
    private String fieldLevelCode;

    //todo 待扩展，进缓存
    public static TreeHelper getTreeHelper(String tableName) {
        return new TreeHelper(tableName);
    }

    //先禁止外部创建
    private TreeHelper(String tableName) {
        this.tableName = tableName;
        table = ModelTableCache.getInstance().getByName(tableName);
        cache = CacheManager.getIntance().getCache(tableName);
        dao = DbEngine.getInstance().findDao(tableName);

        ModelField field = table.findFieldByType(SwEnum.FieldType.PARENT_ID.value);
        fieldParent = field.getName();
        field = table.findFieldByType(SwEnum.FieldType.LEVEL_CODE.value);
        fieldLevelCode = field.getName();
    }

    public long getParentId(T bean) {
        return bean.getLong(fieldParent);
    }

    public String getLevelCode(T bean) {
        return bean.getStr(fieldLevelCode);
    }

    public Collection<T> getChildren(T bean) {
        return getChildren(bean.getEntityId());
    }

    public Collection<T> getChildren(long id) {
        return cache.getListByKey(SwConsts.KEY_PARENT_ID, String.valueOf(id));
    }

    public List<T> getChildren(long id, Comparator<T> comparator) {
        Collection<T> set = cache.getListByKey(SwConsts.KEY_PARENT_ID, String.valueOf(id));
        List<T> list = new ArrayList<>(set);
        if (comparator != null)
        list.sort(comparator);
        return list;
    }

    public List<T> resetTreeLevel(T bean) throws Exception {
        List<T> list = new ArrayList<>();
        if (table.getType() != SwEnum.TableType.TYPE_TREE.value) return list;
        if (!table.isNeedCache()) throw new SwException("请定义为需要缓存！");

        T oldBean = cache.get(bean.getEntityId());
        if (bean.getLong(fieldParent) != oldBean.getLong(fieldParent)) {
            resetParentChildren(bean, list);
        }
        return list;
    }

    private void resetParentChildren(T bean, List<T> list) throws Exception {
        T parent = cache.get(getParentId(bean));
        if (parent != null) {
            bean.put(fieldLevelCode, getLevelCode(parent) + SwConsts.SPLIT_CHAR + parent.getEntityId());
            dao.updateEntity(bean, fieldLevelCode);
            list.add(bean);
        }
        Collection<T> children = getChildren(bean);
        if (children != null && !children.isEmpty()) {
            for (T child : children) {
                resetParentChildren(child, list);
            }
        }
    }
}
